
from .plugins import pm, get_plugins, load_plugins, install_plugin
import argparse


def show_help(args):
    print("vidtoolz update")


class CustomHelpFormatter(argparse.HelpFormatter):
    def _format_action(self, action):
        if isinstance(action, argparse._SubParsersAction):
            return "\n".join(f" {name:<20} {sub.description}" for name, sub in action.choices.items()) + "\n"
        return super()._format_action(action)


def install_cmd(args):
    install_plugin(args.packages, args.upgrade, args.editable, args.force_reinstall, args.no_cache_dir)

def main():
    parser = argparse.ArgumentParser(description="Video Tools for editing videos using windows",
                                     formatter_class=CustomHelpFormatter)
    parser.set_defaults(func=show_help)
    
    subparser = parser.add_subparsers(dest="command")
    
    # add plugins command
    pugs_p = subparser.add_parser("plugins", description="Get all listed plugins")
    pugs_p.set_defaults(func=get_plugins)

    install_parser = subparser.add_parser('install', description="Install plugins in the same environemnt as vidtoolz")
    install_parser.add_argument('--upgrade', action='store_true')
    install_parser.add_argument('--editable', help='Edit mode for packages')
    install_parser.add_argument('--force-reinstall', action='store_true')
    install_parser.add_argument('--no-cache-dir', action='store_true')
    install_parser.add_argument('packages', nargs='+')
    install_parser.set_defaults(func=install_cmd)


    load_plugins()
    

    pm.hook.register_commands(subparser=subparser)
    args = parser.parse_args()

    if args.command:
        args.func(args)
    else:
        parser.print_help()

if __name__ == "__main__":
    main()



import pluggy
from .hookspecs import vidtoolzSpec
import pkg_resources
import sys
from runpy import run_module

pm = pluggy.PluginManager("vidtoolz")
pm.add_hookspecs(vidtoolzSpec)

def load_plugins2():
    if not getattr(sys, "_called_from_test", False):
        for entry_point in pkg_resources.iter_entry_points("vidtoolz_plugins"):
            plugin=entry_point.load()
            pm.register(plugin)

def load_plugins():
    if not getattr(sys, "_called_from_test", False):
        # Only load plugins if not running tests
        pm.load_setuptools_entrypoints("vidtoolz_plugins")

def get_plugins(args):
    plugins = []
    if not getattr(sys, "_called_from_test", False):
        plugin_to_distinfo = dict(pm.list_plugin_distinfo())
        for plugin in pm.get_plugins():
            plugin_info = {
                "name": plugin.__name__,
                "hooks": [h.name for h in pm.get_hookcallers(plugin)],
            }
            distinfo = plugin_to_distinfo.get(plugin)
            if distinfo:
                plugin_info["version"] = distinfo.version
                plugin_info["name"] = distinfo.project_name
            plugins.append(plugin_info)

    if plugins:
        print("Installed Plugins:")
        for p in plugins:
            print(p["name"])
    else:
        print("No external plugins in env.")
    return plugins


def install_plugin(packages, upgrade, editable, force_reinstall, no_cache_dir):
    """Install packages from PyPI into the same environment as vidtoolz"""
    args = ["pip", "install"]
    if upgrade:
        args += ["--upgrade"]
    if editable:
        args += ["--editable", editable]
    if force_reinstall:
        args += ["--force-reinstall"]
    if no_cache_dir:
        args += ["--no-cache-dir"]
    args += list(packages)
    sys.argv = args
    run_module("pip", run_name="__main__")



from setuptools import setup, find_packages
import os

def get_long_description():
    with open(
        os.path.join(os.path.dirname(os.path.abspath(__file__)), "README.md"),
        encoding="utf8",
    ) as fp:
        return fp.read()

setup(
    name="vidtoolz",
    version="0.0.1",
    description="CLI tools for editing videos in python. A plugin based approach.",
    packages=find_packages(),
    long_description=get_long_description(),
    long_description_content_type="text/markdown",
    entry_points={"console_scripts": ["vid = vidtoolz.cli:main"]},
    install_requires =["pluggy","moviepy"],
    python_requires=">=3.9",
    author="Sukhbinder Singh",
    url="https://github.com/sukhbinder/vidtoolz",
    extra_require={
      "test":["pytest"],  
    },
    project_urls={
        "Documentation": "https://sukhbinder.wordpress.com",
        "Issues": "https://github.com/sukhbinder/vidtoolz/issues",
        "CI": "https://github.com/sukhbinder/vidtoolz/actions",
        "Changelog": "https://github.com/sukhbinder/vidtoolz/releases",
    },
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "Intended Audience :: End Users/Desktop",
        "Topic :: Database",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: 3.13",
    ],

    )



from pluggy import HookimplMarker
from pluggy import HookspecMarker


hookspec = HookspecMarker("vidtoolz")
hookimpl = HookimplMarker("vidtoolz")


class vidtoolzSpec:
    @hookspec
    def register_commands(subparser):
        """Register additional CLI sub commands, e.g. 'vidtoolz mycommand ...'"""
